# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.3.0

import enum

import winrt.system
from winrt import _winrt_windows_media_playlists

__all__ = [
    "PlaylistFormat",
    "Playlist",
]

class PlaylistFormat(enum.IntEnum):
    WINDOWS_MEDIA = 0
    ZUNE = 1
    M3U = 2

Playlist = _winrt_windows_media_playlists.Playlist
